package examples.maedchen;

import helperclasses.*;
import ttclasses.*;
import java.util.*;

/*****
 * The <tt>ClashConstraint</tt> class manages constraints of the type
 * "no clashes allowed", i.e. it reports a constraint violation if there
 * are one or more clashes among the events of the resource specified
 * by <tt>owner</tt>.
 *****/
public class ClashConstraint extends HardConstraint {
  private Random random;

  public ClashConstraint() {
    name = "ClashConstraint";
    random = GlobalObjects.random;
  }

  public ConstraintViolation compute(PlanningClass owner) {
    try {
      int penalty = 0;
      ArrayList info = new ArrayList();
      List assignments = ((Resource)owner).getAssignments();
      for (int j = 0; j < assignments.size() - 1; j++) {
        Assignment ass1 = (Assignment)assignments.get(j);
        for (int k = j + 1; k < assignments.size(); k++) {
          Assignment ass2 = (Assignment)assignments.get(k);
          if (ass1.event.clashes(ass2.event)) {
            penalty += 20;
            info.add(ass1.resList);
            info.add(ass2.resList);
            ass1.event.penaltyPoints += 10;
            ass2.event.penaltyPoints += 10;
          }
        }
      }
      if (penalty > 0)
        return new ConstraintViolation(this, owner, "Clash at " + owner, penalty, info);
      return null;
    }
    catch (Exception e) {
      return null;
    }
  } // end of compute(PlanningClass)

  public boolean repair(ConstraintViolation violation) {
    ArrayList info = (ArrayList)violation.information;
    ResourceList rl = (ResourceList)info.get(random.nextInt(info.size()));
    rl.freeResource((Resource)violation.owner);
    return true;
  }

} // end of class ClashConstraint

