package examples.maedchen;

import helperclasses.*;
import ttclasses.*;
import java.util.*;

/*****
 * <i>GirlsAndTechnology</i> <br> This constraint has the duty to assure that no girl
 * is assigned to two different projects of the same type.
 *****/
public class DoubleSignatureConstraint extends HardConstraint {
  Random random;

  public DoubleSignatureConstraint() {
    name = "DoubleSignatureConstraint";
    random = GlobalObjects.random;
  }

  public ConstraintViolation compute(PlanningClass owner) {
    int j, k;
    int penalty = 0;
    ArrayList info = new ArrayList();
    List assignments = ((Girl)owner).getAssignments();
    for (j = 0; j < assignments.size(); j++) {
      Assignment ass1 = (Assignment)assignments.get(j);
      Project p1 = (Project)ass1.event;
      for (k = j + 1; k < assignments.size(); k++) {
        Assignment ass2 = (Assignment)assignments.get(k);
        Project p2 = (Project)ass2.event;
        if (p1.signature.equals(p2.signature)) {
          penalty += 20;
          info.add(ass2.resList);
          ass1.event.penaltyPoints += 10;
          ass2.event.penaltyPoints += 10;
        }
      }
    }
    if (penalty > 0)
      return new ConstraintViolation(this, owner, "Duplicate project assignment for " + owner, penalty, info);
    else
      return null;
  }

  public boolean repair(ConstraintViolation violation) {
    ArrayList info = (ArrayList)violation.information;
    ResourceList rl = (ResourceList)info.get(random.nextInt(info.size()));
    rl.freeResource((Resource)violation.owner);
    return true;
  }

} // end of class DoubleSignatureConstraint
