package examples.school;

import ga.*;
import helperclasses.*;
import ttclasses.*;
import java.io.*;
import java.util.*;


/*****
 * This class implements a real-world school timetabling problem based on the
 * timetabling problem presented by Norbert Oster in his Studienarbeit (2001).
 * One possible instance is the Adam Kraft Gymnasium (databases/xml/KraftGymnasium.xml).
 *****/
public class School extends TTProblem {
  String dataFile;

  /** Creates new <tt>School</tt>, initializes local data with what can be found in <tt>dataDir</tt. */
  public School(String xmlFile) {
    random = GlobalObjects.random;
    dataFile = xmlFile;

    constraints.add(new ClashConstraint());
    constraints.add(new RoomSizeConstraint());
    constraints.add(new PreferenceConstraint());

    initialize(xmlFile);
  } // end of School(String)

  public final TTProblem createNewInstance() {
    return new School(dataFile);
  } // end of createNewInstance()
  
  /**
   * Since all events and resources can be created dynamically while processing
   * the XML data file belonging to the GTL description of the problem, <tt>TTProblem</tt>
   * must be able to dynamically instantiate objects of certain types.
   */
  public Event createEvent(String type, String name) {
    Event result = null;
    if (type.equals("Lesson"))
      result = new Lesson(name);
    if (result != null)
      eventList.add(result);
    return result;
  } // end of createEvent(String, String)

  /**
   * Since all events and resources can be created dynamically while processing
   * the XML data file belonging to the GTL description of the problem, <tt>TTProblem</tt>
   * must be able to dynamically instantiate objects of certain types.
   */
  public Resource createResource(String type, String name) {
    Resource result = null;
    List rt = getResourceType(type);
    if (rt == null)
      rt = addResourceType(type);
    if (type.equals("Clazz"))
      result = new Clazz(name);
    else if (type.equals("Subject"))
      result = new Subject(name);
    else if (type.equals("Room"))
      result = new Room(name);
    else if (type.equals("Teacher"))
      result = new Teacher(name);
    else
      result = super.createResource(type, name);
    if (result != null)
      rt.add(result);
    return result;
  } // end of createResource(String, String)

  public static void main(String args[]) {
    System.out.println("Which timetable shall be created?");
    System.out.println("(1) Adam-Kraft-Gymnasium");
    System.out.print("> ");
    try {
      BufferedReader br = new BufferedReader(new InputStreamReader(System.in));
      String selection = br.readLine();
      switch (Integer.parseInt(selection)) {
        case 1: selection = "../databases/xml/KraftGymnasium.xml"; break;
        default: selection = null;
      }
      School s = new School(selection);
      s.solve(new GeneticAlgorithm());
    }
    catch (Exception e) {
      System.out.println("Something went wrong.");
    }
  } // end of main(String[])

} // end of class Hospital


