/**
 * Implementation of class "GlobalObjects".
 *
 * Author: Stefan Buettcher
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 **/

package helperclasses;

import java.util.*;

/*****
 * The class <tt>GlobalObjects</tt> is supposed to bear some objects that
 * have to be accessed from many different points within the Timetabling GA.
 *****/
public final class GlobalObjects {

  /**
   * Every random number at any place inside the Genetic Algorithm or
   * Timetabling problem (or somewhere else!) is encouraged to use this
   * instance of the random number generator. <br>
   * The reason is simple: The current version of the JVM does not take
   * its random numbers from the Operating System's entropy pool, but
   * initializes with the computer's clock status. That policy would
   * obviously cause problems when hundreds of <tt>Random</tt> instances
   * were created within the same milli-second. The resulting problem
   * is avoided by this single <tt>Random</tt> object.
   **/
  public static Random random = new Random();

  public static long lastTimeCheck = System.currentTimeMillis();

  public static int callCounter = 0;

} // end of class GlobalObjects

