/**
 * Implementation of class "God".
 *
 * Author: Stefan Buettcher
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 **/

package helperclasses;

import examples.*;
import ga.*;
import ttclasses.*;
import java.lang.reflect.*;
import java.util.*;

import examples.maedchen.*;
import examples.hospital.*;


/*****
 * The class <tt>God</tt> manages the initial creation process.
 * It can be used to create populations of different types of
 * individuals. <br>
 * Of course, there is only one instance of <tt>God</tt> in
 * the known universe, so all methods are <tt>static</tt>.
 *****/
public final class God {

  public God() throws BlasphemyException {
    throw new BlasphemyException();
  }

  /**
   * <tt>createPopulation</tt> creates a new population consisting of <tt>size</tt> individuals
   * of the given species (class).
   **/
  public static List createPopulation(String species, String data, int size) {
    List population = new ArrayList();
    GeneticIndividual gi = createIndividual(species, data);
    if (gi != null)
      for (int i = 0; i < size; i++) {
        population.add(gi.duplicate());
}
    return population;
  } // end of createPopulation(String, int)

  /** Creates a new individual of the species specified by <tt>specied</tt>. **/
  public static GeneticIndividual createIndividual(String species, String data) {
    // first, try to create predefined stuff
    if (species.equals("GaT-2000"))
      return new TTChromosome(new MaedchenUndTechnik("../databases/maedchen/2000/Kurse.txt",
                                                     "../databases/maedchen/2000/Kursplan.txt",
                                                     "../databases/maedchen/2000/Teilnehmer.txt"));
    if (species.equals("GaT-2001"))
      return new TTChromosome(new MaedchenUndTechnik("../databases/maedchen/2001/Kurse.txt",
                                                     "../databases/maedchen/2001/Kursplan.txt",
                                                     "../databases/maedchen/2001/Teilnehmer.txt"));
    if (species.equals("GaT-2002"))
      return new TTChromosome(new MaedchenUndTechnik("../databases/maedchen/2002/Kurse.txt",
                                                     "../databases/maedchen/2002/Kursplan.txt",
                                                     "../databases/maedchen/2002/Teilnehmer.txt"));
    else {
      // create by using <tt>species</tt> as class name
      try {
        Class clazz = Class.forName(species);
        Class array[] = new Class[1];
        array[0] = new String().getClass();
        Constructor cons = clazz.getConstructor(array);
        String array2[] = new String[1];
        array2[0] = data;
        TTProblem problem = (TTProblem)cons.newInstance(array2);
        return new TTChromosome(problem);
      } catch (Exception e) {
        System.err.println("Could not create individual of species \"" + species + "\": " + e.toString());
        e.printStackTrace(System.err);
        return null;
      }
    }
  } // end of createIndividual(String, String)

} // end of class God


/*****
 * A <tt>BlasphemyException</tt> is thrown as soon as anybody tries
 * to create his/her own instance of <tt>God</tt>.
 *****/
class BlasphemyException extends Exception {
  
  public String toString() {
    return "Du sollst keine anderen Gtter haben neben mir!";
  }
  
} // end of class BlasphemyException

