package tools;

import java.util.*;
import java.io.*;

/*****
 * The class <tt>TTtoHTML</tt> can be used in order to produce human-readable
 * HTML timetables from the XML output generated by the GA. <br>
 * This class has been written ad-hoc and is very sensitive to changes in the
 * XML structure of the timetable definition (input) file. It is not supposed
 * to be an XML parser! <br>
 * And: There is almost no documentation for this class. So, look out!
 *****/
public class TTtoHTML {
  private String inFile, template, outFile;
  private List timeslots, resources, events;
  private int xExt, yExt;
  
  public TTtoHTML(String inFile, String template, String outFile) {
    this.inFile = inFile;
    this.template = template;
    this.outFile = outFile;
    timeslots = new ArrayList();
    resources = new ArrayList();
    events = new ArrayList();
  } // end of TTtoHTML(String, String, String)

  public void run() {
    processTemplate();
    processInputFile();
    generateOutput();
  } // end of run()

  private void generateOutput() {
    int j, k, l;
    try {
      FileOutputStream fo = new FileOutputStream(outFile);
      PrintStream ps = new PrintStream(fo);
      ps.println("<html><head><title>Timetable</title></head>");
      ps.println("<body bgcolor=\"#FFFFFF\" text=\"#000000\">");
      ps.println("<br> <b>General View: Event Timetable</b> <br>");
      ps.println("<table border=3 cellpadding=2 cellspacing=2>");
      for (j = 1; j <= yExt; j++) {
        ps.println("  <tr>");
        for (k = 1; k <= xExt; k++) {
          ps.println("    <td valign=top>");
          InternalTimeslot slot = getTimeslot(k, j);
          if (slot != null) {
            ps.println("      <p align=right><b>" + slot.name + "</b></p>");
            for (l = 0; l < slot.events.size(); l++) {
              InternalEvent ev = getEvent((String)slot.events.get(l));
if (ev == null) System.err.println("Event " + (String)slot.events.get(l) + " not found.");
              ps.println("      " + ev.toHREF() + " <br>");
            }
          }
          else {
            ps.println("      <p align=right>(empty)</p>");
          }
          ps.println("    </td>");
        }
        ps.println("  </tr>");
      }
      ps.println("</table>");
      
      // create TTs for all resources
      for (j = 0; j < resources.size(); j++) {
        int x, y;
        InternalResource res = (InternalResource)resources.get(j);
        ps.println("<br> <b>Timetable for " + res.name + "</b> <br><br>");
        ps.println("<table border=3 cellspacing=2 cellpadding=2>");
        String[][] timetable = new String[xExt + 1][yExt + 1];
        for (y = 1; y <= yExt; y++)
          for (x = 1; x <= xExt; x++)
            timetable[x][y] = "";
        for (int i = 0; i < res.events.size(); i++) {
          InternalEvent ev = getEvent((String)res.events.get(i));
          for (int z = 0; z < ev.timeslots.size(); z++) {
            InternalTimeslot slot = getTimeslot((String)ev.timeslots.get(z));
            timetable[slot.x][slot.y] += ev.toHREF() + "<br>";
          }
        }
        for (y = 1; y <= yExt; y++) {
          ps.println("  <tr>");
          for (x = 1; x <= xExt; x++) {
            ps.println("    <td valign=top>");
            InternalTimeslot slot = getTimeslot(x, y);
            if (slot == null)
              ps.println("      <p align=right>(empty)</p>");
            else
              ps.println("      <p align=right><b>" + slot.name + "</b></p>");
            ps.println("      " + timetable[x][y]);
            ps.println("    </td>");
          }
          ps.println("  </tr>");
        }
        ps.println("</table>");
          
      } // end of resources TT loop

      ps.println("</body>");
      ps.println("</html>");
    }
    catch (IOException ioe) {
      System.err.println("Error: Could not generate output.");
    }
  } // end of generateOutput()
    
  private void processInputFile() {
    try {
      FileInputStream fi = new FileInputStream(inFile);
      InputStreamReader is = new InputStreamReader(fi);
      BufferedReader br = new BufferedReader(is);
      String line, name;
      int j, k;
      while ((line = br.readLine()) != null) {
        if (line.startsWith("<resource ")) {
          line = line.substring(10).trim();
          if (line.startsWith("name=\"")) {
            line = line.substring(6);
            j = line.indexOf('"');
            if (j <= 0) throw new IOException("Error!");
            name = line.substring(0, j);
            line = line.substring(j + 1);
            if (isTimeslot(name)) {  // process timeslot data
              do {
                line = br.readLine().trim();
              } while (!line.startsWith("<list name=\"events\" content=\""));
              line = line.substring(29);
              j = line.indexOf('"');
              if (j < 0) throw new IOException("Error!");
              line = line.substring(0, j);
              addEventsToTimeslot(name, line);
            }
            else { // consider this an ordinary resource
              do {
                line = br.readLine().trim();
              } while (!line.startsWith("<list name=\"events\" content=\""));
              line = line.substring(29);
              j = line.indexOf('"');
              if (j < 0) throw new IOException("Error!");
              line = line.substring(0, j);
              InternalResource res = new InternalResource(name, stringToList(line));
              resources.add(res);
            }
          }
          do {
            line = br.readLine();
          } while (!line.trim().equals("</resource>"));
        }
        else if (line.startsWith("<event ")) {
          line = line.substring(7).trim();
          if (line.startsWith("name=\"")) {
            line = line.substring(6);
            j = line.indexOf('"');
            if (j <= 0) throw new IOException("Error!");
            name = line.substring(0, j);
            line = line.substring(j + 1);
            do {
              line = br.readLine().trim();
            } while (!line.startsWith("<list name=\"TimeSlots\""));
            j = line.indexOf("content=\"");
            if (j <= 0) throw new IOException("Error!");
            line = line.substring(j + 9);
            j = line.indexOf('"');            
            if (j < 0) throw new IOException("Error!");
            line = line.substring(0, j);
            events.add(new InternalEvent(name, stringToList(line)));
          }
          
          do {
            line = br.readLine();
          } while (!line.trim().equals("</event>"));
        }
      }
    }
    catch (IOException ioe) {
      System.err.println("Error: Could not process input file.");
    }
  } // end of processInputFile()
  
  private void processTemplate() {
    try {
      FileInputStream fi = new FileInputStream(template);
      InputStreamReader is = new InputStreamReader(fi);
      BufferedReader br = new BufferedReader(is);
      String line, name;
      int j, k;
      while ((line = br.readLine()) != null) {
        if (line.startsWith("timeslots=")) {
          line = line.substring(10);
          j = line.indexOf('*');
          if (j > 0) {
            xExt = Integer.parseInt(line.substring(0, j));
            yExt = Integer.parseInt(line.substring(j + 1));
          }
          else {
            xExt = Integer.parseInt(line);
            yExt = 1;
          }
        }
        else if (line.startsWith("\"")) {
          j = line.indexOf('=');
          name = line.substring(0, j);
          line = line.substring(j + 1);
          if ((name.charAt(0) == '"') && (name.charAt(name.length() - 1) == '"')) {
            name = name.substring(1, name.length() - 1);
            j = line.indexOf(',');
            if (j > 0) {
              k = Integer.parseInt(line.substring(j + 1));
              j = Integer.parseInt(line.substring(0, j));
            }
            else {
              j = Integer.parseInt(line);
              k = 1;
            }
            timeslots.add(new InternalTimeslot(name, j, k));
          }
        }
      }
    }
    catch (IOException ioe) {
      System.err.println("Error: Could not process template file.");
    }
  } // end of processTemplate

  /* Tells the caller if the resource with the given name is a timeslot. */
  private boolean isTimeslot(String name) {
    return (getTimeslot(name) != null);
  } // end of isTimeslot(String)

  private List stringToList(String stuff) {
    List result = new ArrayList();
    int j;
    while ((j = stuff.indexOf(';')) >= 0) {
      result.add(stuff.substring(0, j));
      stuff = stuff.substring(j + 1);
    }
    result.add(stuff);
    return result;
  } // end of stringToList(stuff)
  
  private InternalTimeslot getTimeslot(String name) {
    int k = timeslots.size();
    for (int i = 0; i < k; i++) {
      InternalTimeslot slot = (InternalTimeslot)timeslots.get(i);
      if (slot.name.equals(name))
        return slot;
    }
    return null;
  } // end of getTimeslot(String)

  private InternalTimeslot getTimeslot(int x, int y) {
    int k = timeslots.size();
    for (int i = 0; i < k; i++) {
      InternalTimeslot slot = (InternalTimeslot)timeslots.get(i);
      if ((slot.x == x) && (slot.y == y))
        return slot;
    }
    return null;
  } // end of getTimeslot(String)
  
  private void addEventsToTimeslot(String name, String events) {
    InternalTimeslot slot = getTimeslot(name);
    if (slot != null)
      slot.events = stringToList(events);
  } // end of addEventsToTimeslot
  
  private InternalEvent getEvent(String name) {
    int k = events.size();
    for (int i = 0; i < k; i++) {
      InternalEvent ev = (InternalEvent)events.get(i);
      if (ev.name.equals(name))
        return ev;
    }
    return null;
  } // end of getEvent(String)
  
  public static void main(String args[]) {
    if (args.length != 3) {
      System.out.println("Syntax: TTtoHTML infile.xml template.tt outfile.html");
    }
    else {
      TTtoHTML html = new TTtoHTML(args[0], args[1], args[2]);
      html.run();
    }
  } // end of main(String[])

} // end of class TTtoHTML

class InternalResource {
  public String name;
  public List events;
  
  public InternalResource(String name, List events) {
    this.name = name;
    this.events = events;
  }
  
} // end of class InternalResource

class InternalEvent {
  public String name;
  public List timeslots;

  public InternalEvent(String aName, List aList) {
    name = aName;
    timeslots = aList;
  }
  
  private String nameToAnchor(String name) {
    return name.toLowerCase().replace(' ', '_');
  } // end of nameToAnchor(String)
  
  public String toHREF() {
    return "<a href=\"" + nameToAnchor(name) + "\">" + name + "</a>";
  }
  
} // end of class InternalEvent

class InternalTimeslot {
  public String name;
  public int x, y;
  public List events;
  
  public InternalTimeslot(String aName, int aX, int aY) {
    name = aName;
    x = aX;
    y = aY;
    events = new ArrayList();
  }
  
} // end of class InternalTimeslot

