package tools.frontend;

import ga.*;

import java.awt.*;
import java.io.*;
import java.util.*;
import java.awt.event.*;
import javax.swing.*;

/**
 * <p>Title: Timetabling GA [Frontend]</p>
 * <p>Description: This is a frontend for the timetabling GA.</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: FAU</p>
 * @author Stefan Buettcer
 * @version 1.0
 */

public final class MainFrame extends JFrame {
  private JPanel contentPane;
  private JMenuBar jMenuBar1 = new JMenuBar();
  private JMenu jMenuFile = new JMenu();
  private JMenuItem jMenuFileExit = new JMenuItem();
  private JMenu jMenuHelp = new JMenu();
  private JMenuItem jMenuHelpAbout = new JMenuItem();
  private JToolBar jToolBar = new JToolBar();
  private JButton jButton1 = new JButton();
  private JButton jButton2 = new JButton();
  private JButton jButton3 = new JButton();
  private ImageIcon image1;
  private ImageIcon image2;
  private ImageIcon image3;
  private JLabel statusBar = new JLabel();
  private BorderLayout borderLayout1 = new BorderLayout();
  private JPanel jPanel1 = new JPanel();
  private JPanel jPanel2 = new JPanel();
  private JPanel jPanel3 = new JPanel();
  private BorderLayout borderLayout2 = new BorderLayout();
  private JLabel status = new JLabel();
  private JButton startButton = new JButton();
  private JButton pauseButton = new JButton();
  private JButton nextButton = new JButton();
  private JButton stopButton = new JButton();
  private JButton htmlButton = new JButton();
  private JButton xmlButton = new JButton();

  public String species, initString;
  private EvolutionThread thread;
  private MainFrame mainFrame;
  private boolean paused;
  
  private Diagram diagram;
  private double avg[], best[];
  private int dataCount;
  
  public String fileName;
  public int fileType;

  // Construct the frame
  public MainFrame() {
    enableEvents(AWTEvent.WINDOW_EVENT_MASK);
    try {
      jbInit();
    }
    catch(Exception e) {
      e.printStackTrace();
    }
    species = initString = null;
    thread = null;
    mainFrame = this;
    
    avg = new double[10000];
    best = new double[10000];
    dataCount = 0;
  }

  //Component initialization
  private void jbInit() throws Exception  {
    image1 = new ImageIcon(MainFrame.class.getResource("openFile.gif"));
    image2 = new ImageIcon(MainFrame.class.getResource("closeFile.gif"));
    image3 = new ImageIcon(MainFrame.class.getResource("help.gif"));
    //setIconImage(Toolkit.getDefaultToolkit().createImage(MainFrame.class.getResource("[Your Icon]")));
    contentPane = (JPanel) this.getContentPane();
    contentPane.setLayout(borderLayout1);
    this.setSize(new Dimension(400, 300));
    this.setTitle("Timetabling GA [Frontend]");
    statusBar.setText(" ");
    jMenuFile.setText("Datei");
    jMenuFileExit.setText("Beenden");
    jMenuFileExit.addActionListener(new ActionListener()  {
      public void actionPerformed(ActionEvent e) {
        jMenuFileExit_actionPerformed(e);
      }
    });
    jMenuHelp.setText("Hilfe");
    jMenuHelpAbout.setText("Info");
    jMenuHelpAbout.addActionListener(new ActionListener()  {
      public void actionPerformed(ActionEvent e) {
        jMenuHelpAbout_actionPerformed(e);
      }
    });
    jButton1.setIcon(image1);
    jButton1.setToolTipText("Neuen Evolutionsvorgang starten");
    jButton1.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        jButton1_actionPerformed(e);
      }
    });
    jButton2.setIcon(image2);
    jButton2.setEnabled(false);
    jButton2.setToolTipText("Speichern");
    jButton2.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        jButton2_actionPerformed(e);
      }
    });
    jButton3.setIcon(image3);
    jButton3.setToolTipText("Info");
    jButton3.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        jMenuHelpAbout_actionPerformed(e);
      }
    });
    jPanel1.setBorder(BorderFactory.createEtchedBorder());
    jPanel1.setMaximumSize(new Dimension(32767, 32));
    jPanel1.setMinimumSize(new Dimension(10, 32));
    jPanel1.setPreferredSize(new Dimension(10, 32));
    jPanel2.setBorder(BorderFactory.createEtchedBorder());
    jPanel2.setMaximumSize(new Dimension(140, 32767));
    jPanel2.setMinimumSize(new Dimension(140, 200));
    jPanel2.setPreferredSize(new Dimension(140, 10));
    jPanel3.setBorder(BorderFactory.createEtchedBorder());
    jPanel3.setLayout(borderLayout2);
    status.setFont(new java.awt.Font("Dialog", 0, 12));
    status.setForeground(Color.black);
    status.setText("Status: unbekannt");
    startButton.setEnabled(false);
    startButton.setMaximumSize(new Dimension(130, 29));
    startButton.setMinimumSize(new Dimension(130, 29));
    startButton.setPreferredSize(new Dimension(130, 29));
    startButton.setText("Start");
    startButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        thread = new EvolutionThread(mainFrame, species, initString);
        thread.start();
        startButton.setEnabled(false);
        pauseButton.setEnabled(true);
        stopButton.setEnabled(true);
        paused = false;
      }
    });
    pauseButton.setEnabled(false);
    pauseButton.setMaximumSize(new Dimension(130, 29));
    pauseButton.setMinimumSize(new Dimension(130, 29));
    pauseButton.setPreferredSize(new Dimension(130, 29));
    pauseButton.setText("Pause");
    pauseButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        if (paused) {
          nextButton.setEnabled(false);
          nextButton.setEnabled(false);
          htmlButton.setEnabled(false);
          xmlButton.setEnabled(false);
          jButton2.setEnabled(false);
          pauseButton.setText("Pause");
          paused = false;
          thread.paused = false;
        }
        else {
          thread.paused = true;
          paused = true;
          pauseButton.setText("Weiter");
          nextButton.setEnabled(true);
          htmlButton.setEnabled(true);
          xmlButton.setEnabled(true);
          jButton2.setEnabled(true);
        }
      }
    });
    nextButton.setEnabled(false);
    nextButton.setMaximumSize(new Dimension(130, 29));
    nextButton.setMinimumSize(new Dimension(130, 29));
    nextButton.setPreferredSize(new Dimension(130, 29));
    nextButton.setText("Nchste Gen.");
    nextButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        thread.paused = false;
        try { Thread.sleep(2); } catch (InterruptedException ie) { }
        thread.paused = true;
      }
    });
    stopButton.setEnabled(false);
    stopButton.setMaximumSize(new Dimension(130, 29));
    stopButton.setMinimumSize(new Dimension(130, 29));
    stopButton.setPreferredSize(new Dimension(130, 29));
    stopButton.setText("Stopp");
    stopButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        thread.terminated = true;
        thread = null;
        stopButton.setEnabled(false);
        pauseButton.setEnabled(false);
        nextButton.setEnabled(false);
        pauseButton.setText("Pause");
        startButton.setEnabled(true);
        status.setText("Spezies gewhlt: " + species);
        dataCount = 0;
      }
    });
    htmlButton.setEnabled(false);
    htmlButton.setMaximumSize(new Dimension(130, 29));
    htmlButton.setMinimumSize(new Dimension(130, 29));
    htmlButton.setPreferredSize(new Dimension(130, 29));
    htmlButton.setText("HTML sehen");
    xmlButton.setEnabled(false);
    xmlButton.setMaximumSize(new Dimension(130, 29));
    xmlButton.setMinimumSize(new Dimension(130, 29));
    xmlButton.setPreferredSize(new Dimension(130, 29));
    xmlButton.setMnemonic('0');
    xmlButton.setText("XML sehen");
    contentPane.setMinimumSize(new Dimension(480, 320));
    contentPane.setPreferredSize(new Dimension(480, 320));
    jToolBar.add(jButton1);
    jToolBar.add(jButton2);
    jToolBar.add(jButton3);
    jMenuFile.add(jMenuFileExit);
    jMenuHelp.add(jMenuHelpAbout);
    jMenuBar1.add(jMenuFile);
    jMenuBar1.add(jMenuHelp);
    this.setJMenuBar(jMenuBar1);
    contentPane.add(jToolBar, BorderLayout.NORTH);
    contentPane.add(statusBar, BorderLayout.WEST);
    contentPane.add(jPanel1, BorderLayout.SOUTH);
    jPanel1.add(status, null);
    contentPane.add(jPanel2, BorderLayout.EAST);
    jPanel2.add(startButton, null);
    jPanel2.add(pauseButton, null);
    jPanel2.add(nextButton, null);
    jPanel2.add(stopButton, null);
    jPanel2.add(htmlButton, null);
    jPanel2.add(xmlButton, null);
    contentPane.add(jPanel3, BorderLayout.CENTER);
    diagram = new Diagram();
    double best[] = new double[2]; best[0] = -55000; best[1] = -51000;
    double avg[] = new double[2]; avg[0] = -53000; avg[1] = -52000;
    diagram.setData(avg, best);
    jPanel3.add(diagram, BorderLayout.CENTER);
  }

  // File | Exit action performed
  public void jMenuFileExit_actionPerformed(ActionEvent e) {
    System.exit(0);
  }

  // Help | About action performed
  public void jMenuHelpAbout_actionPerformed(ActionEvent e) {
    MainFrame_AboutBox dlg = new MainFrame_AboutBox(this);
    Dimension dlgSize = dlg.getPreferredSize();
    Dimension frmSize = getSize();
    Point loc = getLocation();
    dlg.setLocation((frmSize.width - dlgSize.width) / 2 + loc.x, (frmSize.height - dlgSize.height) / 2 + loc.y);
    dlg.setModal(true);
    dlg.pack();
    dlg.show();
  }

  //Overridden so we can exit when window is closed
  protected void processWindowEvent(WindowEvent e) {
    super.processWindowEvent(e);
    if (e.getID() == WindowEvent.WINDOW_CLOSING) {
      jMenuFileExit_actionPerformed(null);
    }
  }

  // InitializeButton pressed
  public void jButton1_actionPerformed(ActionEvent e) {
    SpeciesChooser dlg = new SpeciesChooser(this);
    Dimension dlgSize = dlg.getPreferredSize();
    Dimension frmSize = getSize();
    Point loc = getLocation();
    dlg.setLocation((frmSize.width - dlgSize.width) / 2 + loc.x, (frmSize.height - dlgSize.height) / 2 + loc.y);
    dlg.setModal(true);
    dlg.pack();
    dlg.show();
    if (initString != null) {
      startButton.setEnabled(true);
      status.setText("Spezies gewhlt: " + species);
    }
    else {
      startButton.setEnabled(false);
      status.setText("Status: unbekannt");
    }
  }

  // SaveButton pressed
  public void jButton2_actionPerformed(ActionEvent e) {
    SaveDialog dlg = new SaveDialog(this);
    Dimension dlgSize = dlg.getPreferredSize();
    Dimension frmSize = getSize();
    Point loc = getLocation();
    dlg.setLocation((frmSize.width - dlgSize.width) / 2 + loc.x, (frmSize.height - dlgSize.height) / 2 + loc.y);
    dlg.setModal(true);
    dlg.pack();
    dlg.show();
    if (fileName != null) {  // Success
      GeneticIndividual checker = thread.ga.getBestIndividual();
      String string = "";
      switch (fileType) {
        case 1: string = checker.getSummary(); break;  // text/plain
        case 2: string = checker.toXML(); break;  // text/xml
        case 3: string = checker.toHTML(); break;  // text/html
      }
      try {
        OutputStream os = new FileOutputStream(fileName);
        os.write(string.getBytes());
        os.flush();
      } catch (IOException ioe) {
        System.out.println("\nAn IOException occurred while writing file to disk.\n");
      }
    }
  }
  
  /** Lets the caller add new entries to the dataset that is displayed. **/
  protected void addData(int genCount, double fitness, double average) {
    status.setText("Generation: " + genCount + ". Bester Fitness-Wert: " + fitness + ".");
    best[dataCount] = fitness;
    avg[dataCount] = average;
    dataCount++;
    if (dataCount >= best.length) {
      double newBest[] = new double[dataCount + 10000];
      double newAvg[] = new double[dataCount + 10000];
      System.arraycopy(best, 0, newBest, 0, dataCount);
      System.arraycopy(avg, 0, newAvg, 0, dataCount);
      best = newBest;
      avg = newAvg;
    }
    double myBest[] = new double[dataCount];
    double myAvg[] = new double[dataCount];
    System.arraycopy(best, 0, myBest, 0, dataCount);
    System.arraycopy(avg, 0, myAvg, 0, dataCount);
    diagram.setData(myBest, myAvg);
    diagram.repaint();
  } // end of addData(int, double)

}

/*****
 * The class <tt>Diagram</tt> paints the temporal development of the
 * population that the GA is evolving.
 *****/
final class Diagram extends Component {
  private double best[], average[];
  private double min, max;
  private int count, width, height;

  private int topMargin, bottomMargin, leftMargin, rightMargin;

  public Diagram() {
    best = average = null;
    topMargin = rightMargin = 20;
    leftMargin = 50;
    bottomMargin = 30;
  }

  public void setData(double[] best, double[] average) {
    min = 123456789;
    max = -12345678;
    count = best.length;
    this.best = new double[count];
    this.average = new double[count];
    for (int i = 0; i < count; i++) {
      this.best[i] = best[i];
      if (best[i] < min) min = best[i];
      if (average[i] < min) min = average[i];
      this.average[i] = average[i];
      if (average[i] > max) max = average[i];
      if (best[i] > max) max = best[i];
    }
  }

  public void drawEmptyBoard(Graphics g) {
    g.setColor(Color.white);
    g.fillRect(0, 0, width, height);
  }

  private void calculateMarks() {
  }

  private int valueToY(double value) {
    value -= min;
    value /= max - min;
    return height - bottomMargin - (int)(value * (height - topMargin - bottomMargin));
  }

  private int countToX(int c) {
    return leftMargin + (int)(c * (width - leftMargin - rightMargin) / count);
  }

  public void paint(Graphics g) {
    int i, x, y;
    width = getWidth();
    height = getHeight();
    drawEmptyBoard(g);
    if (best == null)
      return;

    calculateMarks();

    FontMetrics fm = g.getFontMetrics();

    // draw axes
    g.setColor(Color.black);
    x = countToX(count);
    y = valueToY(min);
    g.drawLine(countToX(0), y, x, y);
    g.drawLine(x, y, x - 5, y - 5);
    g.drawLine(x, y, x - 5, y + 5);
    g.drawString("Generationen", x - fm.stringWidth("Generationen") + 10,
                 y + fm.getHeight());
    x = countToX(0);
    y = valueToY(max);
    g.drawLine(x, valueToY(min), x, y);
    g.drawLine(x, y, x - 5, y + 5);
    g.drawLine(x, y, x + 5, y + 5);
    g.drawString("Fitness-Werte", x - fm.stringWidth("Fitness-Werte")/2, y - 5);

    if (count >= 2) {
      g.setColor(Color.red);
      for (i = 1; i < count; i++)
        g.drawLine(countToX(i - 1), valueToY(average[i - 1]),
                   countToX(i), valueToY(average[i]));
      g.setColor(Color.blue);
      for (i = 1; i < count; i++)
        g.drawLine(countToX(i - 1), valueToY(best[i - 1]),
                   countToX(i), valueToY(best[i]));
    }
  }

} // end of class Diagram
